<?php
//module/Secure/src/Secure/Controller/ForgetpassController.php

/**************
	Purpose: Implements the forget password functionality for HTC.
	Author: Kevin Gyberg
	Date: 02/20/2013
*************/

namespace Login\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\EventManager\EventManagerInterface;
use Zend\Session\Container;
use Login\Form\ForgetpassForm;
use Login\Model\Forgetpass;
use Zend\Mail;
use Zend\Mime\Part as MimePart;
use Zend\Mime\Message as MimeMessage;

class ForgetpassController extends AbstractActionController
{
	/**
     * @var authservice
     */
	protected $authservice;

    /**
     * @var convert
     */
    protected $convert;

    /**
     * @var config
     */
    protected $config;
    
    /**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
        // Assign all array of constants to the constantArray variable.
        $constantArray = $this->config;

		$this->sessionInstance = new Container($constantArray['SESSION_CONTAINER_NAME']);
		return $this->sessionInstance;
	}
    
	/**
     * Gets an instance of authentication service.
     *
     * @return instance of authservice
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()
                                      ->get('AuthService');
        }
        
        return $this->authservice;
    }
    
    /**
     * Inject an EventManager instance
     *
     * @param  EventManagerInterface $eventManager
     * @return void
     */
    public function setEventManager(EventManagerInterface $events)
    {
        parent::setEventManager($events);
 
        $controller = $this;
        $events->attach('dispatch', function ($e) use ($controller) {
            
            $requestTrackerArray = $controller->createRequestTrackerRecord();
            
            $controller
                ->getRequestTrackerTable()
                ->saveRequestTrackerRecord($requestTrackerArray);
            
        }, 100); // execute before executing action logic
 
        return $this;
    }
    
    /**
     * Function to process the request the
     * 
     * @return type
     */
    public function createRequestTrackerRecord()
    {   
        // array to return after processing
        $requestTrackerArray = array();
        
        // call the convert param service to encryption and decryption as required.
        $this->convert = $this->getServiceLocator()->get('ConvertParams');
        $this->config = $this->getServiceLocator()->get('Config');
        
        // get the constant array
        $constantArray = $this->config;
        
        // initialize the session array.
        $session = $this->intializeSessionInstance();
        $isLoggedIn = $session->offsetGet('is_logged_in');
        
        // check if the request uri key of $_SERVER is not empty.
        if (! empty($_SERVER['REQUEST_URI'])) {
            
            $requestTrackerArray['http_user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $requestTrackerArray['script_uri'] = $_SERVER['SCRIPT_NAME'];
            $requestTrackerArray['request_uri'] = $_SERVER['REQUEST_URI'];
            $requestTrackerArray['domain_name'] = $_SERVER['SERVER_NAME'];
            $requestTrackerArray['remote_ipadd'] = $_SERVER['REMOTE_ADDR'];
            $requestTrackerArray['http_referrer'] =  $_SERVER['HTTP_REFERER'];
            $requestTrackerArray['resource_action'] 
                                        = $constantArray['account']['UPDATE_ACTION'];
            
            if ($isLoggedIn === $constantArray['IS_LOGGED_IN_YES']) {

                $requestTrackerArray['session_userid'] = $session->offsetGet('user_id');
            }
            
            $parseArray = parse_url($_SERVER['HTTP_REFERER']);
            $requestTrackerArray['ipaddr_hostname'] = gethostbyname($parseArray['host']);
            $requestTrackerArray['created'] = date('Y-m-d H:i:s');
            $requestTrackerArray['created_date'] = date('Y/m/d', time());
            $requestTrackerArray['created_time'] = date('H:i:s', time());
        }
        
        return $requestTrackerArray;
    }
    
    /**
     * Renders the forget password form and resets the password for entered email id.
     *
     * @return array(Forget password form object).
     */
	function indexAction()
    {   
		// get an instance of register form
		$form = new ForgetpassForm();

        // Assign all the group of constants to the constantArray.
        $constantArray = $this->config;

		$isLoggedIn = $constantArray['$constantArray'];
		
		// check if the user is already logged in.
		if ($this->getAuthService()->hasIdentity()) {

			return $this->redirect()->toRoute('secure');
		}
		
		$this->getEvent()->getViewModel()->setVariables(
            array(
                'is_logged_in'           => $isLoggedIn,
                'noBetaFeedbackLink'     => TRUE
            )
        );
		
		// check if the request is a post request
        if ($this->getRequest()->isPost()) {
			$isAlreadyPresent = false;
			$user = new Forgetpass();
            $form->setInputFilter($user->getInputFilter());
            $form->setData($this->getRequest()->getPost());
			
			// apply validations on the submitted form
            if ($form->isValid()) {
				
				$emailAddress = strtolower($this->getRequest()->getPost()->email_address);
                $emailExists = $this
                                    ->getUsersTable()
                                    ->ifEmailIdExists($emailAddress);

                // check if the email id pre - exists in the system.
				if ($emailExists) {
					$isAlreadyPresent = true;
					$greetingSrv = $this->getServiceLocator()->get('PasswordGenerator');
					// generate a random password
					$randomPassword = $greetingSrv->createPassword();
					$encryptedPassword = md5($randomPassword);
					
					// reset the database
					$userData = $this
									->getUsersTable()
									->resetPassword(
                                            $emailExists->userid, 
                                            $encryptedPassword
                    );
					
					// if the password is updated properly
					if ($userData) {
					
						// send and email to user.
						$options = new Mail\Transport\SmtpOptions(
                                $constantArray['mailer']['smtp_options']
                        );
                        
						$mailData = array(
							'firstname'		=>		$userData->first_name,
							'lastname'		=>		$userData->last_name,
							'email_address' => 		$emailAddress,
							'password' 		=> 		$randomPassword
						);
						$this->renderer = $this->getServiceLocator()->get('ViewRenderer');  
						$content = $this->renderer->render('email/view', $mailData);  
						  
						// make a header as html  
						$html = new MimePart($content);  
						$html->type = "text/html";  
						$body = new MimeMessage();  
						$body->setParts(array($html,)); 
						  
						// instance mail   
						$mail = new Mail\Message();  
						$mail->setBody($body); // will generate our code html from template.phtml  
						$mail->setFrom(
                            $constantArray['mailer']['password_reset_mail_constants']['from_email'],
                            $constantArray['mailer']['password_reset_mail_constants']['from_name']
                        );  
						$mail->setTo($emailAddress);
						$mail->setSubject(
                            $constantArray['mailer']['password_reset_mail_constants']['subject']
                        );
                        
						$transport = new Mail\Transport\Smtp($options);
						$transport->send($mail);
                        return array(
							'form' => $form,
							'is_email_id_present' => $isAlreadyPresent
						);
					}
					else {
						// unable to reset the password
						// send validation message unable to reset the password
						return array(
							'form' => $form,
							'message' => 'Unable to reset password.',
							'is_email_id_present' => $isAlreadyPresent
						);
					}
				}
				else {
					// email id does not exist.
					return array(
						'form' => $form, 
						'is_email_id_present' => $isAlreadyPresent
					);
				}
			}
		}
		return array('form' => $form);
	}
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
    
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getRequestTrackerTable()
	{
		// check if usersTable previously defined.
		if (!$this->requestTrackerTable) {
			$sm = $this->getServiceLocator();
			$this->requestTrackerTable = $sm->get('Secure\Model\RequestTrackerTable');
		}
		return $this->requestTrackerTable;
	}
    
    
}