<?php
//module/Secure/src/Secure/Controller/LoginController.php

/**************
	Purpose: Implements the authentication of user to HTC
	Author: Kevin Gyberg
	Date: 02/20/2013
*************/

namespace Login\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\EventManager\EventManagerInterface;
use Zend\Form\Annotation\AnnotationBuilder;
use Zend\View\Model\ViewModel;
use Login\Model\User;
use Login\Model\Users;
use Login\Model\UsersTable;
use Zend\Session\Container;
use Zend\Filter\StripTags;
use Zend\Filter\StringTrim;

class LoginController extends AbstractActionController
{
	/**
     * @var form
     */
    protected $form;
	
	/**
     * @var storage
     */
    protected $storage;
	
	/**
     * @var authservice
     */
    protected $authservice;
    
    /**
     * @var config
     */
	protected $config;
    
	/**
     * Gets an instance of authentication service.
     *
     * @return instance of authservice
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()->get('AuthService');
        }
		
        return $this->authservice;
    }
    
    /**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
        // Assign all array of constants to the constantArray variable.
        $constantArray = $this->config;

		$this->sessionInstance = new Container($constantArray['SESSION_CONTAINER_NAME']);
		return $this->sessionInstance;
	}
    
    /**
     * Inject an EventManager instance
     *
     * @param  EventManagerInterface $eventManager
     * @return void
     */
    public function setEventManager(EventManagerInterface $events)
    {
        parent::setEventManager($events);
 
        $controller = $this;
        $events->attach('dispatch', function ($e) use ($controller) {
            
            $requestTrackerArray = $controller->createRequestTrackerRecord();
            
            $controller
                ->getRequestTrackerTable()
                ->saveRequestTrackerRecord($requestTrackerArray);
            
        }, 100); // execute before executing action logic
 
        return $this;
    }
    
    /**
     * Function to process the request the
     * 
     * @return type
     */
    public function createRequestTrackerRecord()
    {   
        // array to return after processing
        $requestTrackerArray = array();
        
        // call the convert param service to encryption and decryption as required.
        $this->convert = $this->getServiceLocator()->get('ConvertParams');
        $this->config = $this->getServiceLocator()->get('Config');
        
        // get the constant array
        $constantArray = $this->config;
        
        // initialize the session array.
        $session = $this->intializeSessionInstance();
        $isLoggedIn = $session->offsetGet('is_logged_in');
        
        // check if the request uri key of $_SERVER is not empty.
        if (! empty($_SERVER['REQUEST_URI'])) {
            
            $requestTrackerArray['http_user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $requestTrackerArray['script_uri'] = $_SERVER['SCRIPT_NAME'];
            $requestTrackerArray['request_uri'] = $_SERVER['REQUEST_URI'];
            $requestTrackerArray['domain_name'] = $_SERVER['SERVER_NAME'];
            $requestTrackerArray['remote_ipadd'] = $_SERVER['REMOTE_ADDR'];
            $requestTrackerArray['http_referrer'] =  $_SERVER['HTTP_REFERER'];
            
            if ($isLoggedIn === $constantArray['IS_LOGGED_IN_YES']) {

                $requestTrackerArray['session_userid'] = $session->offsetGet('user_id');
            }
            
            $parseArray = parse_url($_SERVER['HTTP_REFERER']);
            $requestTrackerArray['ipaddr_hostname'] = gethostbyname($parseArray['host']);
            $requestTrackerArray['created'] = date('Y-m-d H:i:s');
            $requestTrackerArray['created_date'] = date('Y/m/d', time());
            $requestTrackerArray['created_time'] = date('H:i:s', time());
        }
        
        return $requestTrackerArray;
    }
    
	/**
     * Gets an instance of auth storage model to access data present in session.
     *
     * @return instance to access session data.
     */
    public function getSessionStorage()
    {
		// check if the storage data member is initialized
        if (! $this->storage)
        {
            $this->storage = $this->getServiceLocator()
                                  ->get('Secure\Model\MyAuthStorage');
        }
        
        return $this->storage;
    }
    
	/**
     * Builds the form using annotation form builder as defined in class User.
     *
     * @return object(Zend\Form\Form)
     */
    public function getForm()
    {
		// check if the form data member is initialized
        if (! $this->form) {
            $user       = new User();
            $builder    = new AnnotationBuilder();
            $this->form = $builder->createForm($user);
        }
       
        return $this->form;
    }
	
	/**
     * Check if user logged in, if not show login form
     *
     * @return array(form instance, flash messages).
     */
    public function loginAction()
    {
        // Assign all group of constants to the constantArray variable.
        $constantArray = $this->config;

		$isLoggedIn = $constantArray['IS_LOGGED_IN_NO'];
		
        //if already login, redirect to secure page 
        if ($this->getAuthService()->hasIdentity()) {

            return $this->redirect()->toRoute('secure');
		}
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'is_logged_in'           => $isLoggedIn,
                    'noBetaFeedbackLink'     => TRUE
                )
        );
        
		// get an instance of the form
        $form       = $this->getForm();
        
        $referrer = '';
        
        if (isset($_GET['referrer']) && ! empty($_GET['referrer']))
        {
            $referrer = $this->filterFormElements($_GET['referrer']);
        }
        
        $activationCode = '';
        
        if (isset($_GET['activation-code']) && ! empty($_GET['activation-code']))
        {
            $activationCode = trim($_GET['activation-code']);
            
            $userIdArray = $this->createDecodedArray($activationCode);
            
            if ($userIdArray['userid']) {
                
                $userRecord = $this->getUsersTable()->getUser($userIdArray['userid']);

                // send an email with the generated activation code to the user's email id.
                $convert = $this->getServiceLocator()->get('SendEmail');

                $activationLink = $constantArray['activation_email_link'] 
                    . $userRecord['activation_code'];

                // prepare the view for activation email template
                $mailData = array(
                   'activationLink'    =>      $activationLink
                );

                $this->renderer = $this->getServiceLocator()->get('ViewRenderer');
                $content = $this->renderer->render('activation-email/view', $mailData);

                // create the link for activation code.
                $smtpOptions = $constantArray['mailer']['smtp_options'];
                $htmlBody = $content;
                $subject = $constantArray['mailer']['registration_mail_constants']['registration_email_subject'];
                $fromName = $constantArray['mailer']['registration_mail_constants']['registration_email_from_name'];
                $fromEmail = $constantArray['mailer']['registration_mail_constants']['registration_email_from_email'];
                $to = $userRecord['email_login'];

                // $htmlBody, $subject, $fromName, $fromEmail, $to
                $convert->sendEmail(
                        $smtpOptions, 
                        $htmlBody, 
                        $subject, 
                        $fromName, 
                        $fromEmail, 
                        $to
                );
            }
        }
        
        return array(
            'form'      => $form,
            'messages'  => $this->flashmessenger()->getMessages(),
            'referrer'  => $referrer
        );
    }
    
	/**
     * Authenticates the entered credentials.
     *
     * @return object(Zend\Http\PhpEnvironment\Response)
     */
    public function authenticateAction()
    {
		// get an instance of login form
        $form       = $this->getForm();
        $redirect = 'login';
        
        $constantArray = $this->config;
        
        $request = $this->getRequest();
		
		// check if the request is a post request.
        if ($request->isPost()) {
            $form->setData($request->getPost());
			
			// apply validations on the submitted form
            if ($form->isValid()) {
			
                // check authentication...
                $this->getAuthService()->getAdapter()
                                       ->setIdentity($request->getPost('username'))
                                       ->setCredential($request->getPost('password'));
                                       
                $result = $this->getAuthService()->authenticate();
                foreach ($result->getMessages() as $message) {
                    
                    // save message temporary into flashmessenger
                    $this->flashmessenger()->addMessage($message);
                }
                
				// apply validations on the submitted form
                if ($result->isValid()) {
                    
                    $redirect = 'secure';
					$userId = $this->getAuthService()
										->getAdapter()
										->getResultRowObject()
										->userid;
					$emailLogin = $this->getAuthService()
										->getAdapter()
										->getResultRowObject()
										->email_login;
					$isAdmin = $this->getAuthService()
										->getAdapter()
										->getResultRowObject()
										->is_admin;
					
					$session = new Container($constantArray['SESSION_CONTAINER_NAME']);
					$session->offsetSet('is_logged_in', $constantArray['IS_LOGGED_IN_YES']);
					$session->offsetSet('user_id', $userId);
					$session->offsetSet('email_login', $emailLogin);
                    $session->offsetSet('is_admin', FALSE);
                    
                    if (! empty($isAdmin)) {
                        $session->offsetSet('is_admin', TRUE);
                        $redirect = 'admin';
                    }
                    
                    $rootFolderId = $this
                                        ->getUsersTable()
                                        ->getRootFolderIdByUserid($userId);
					$session->offsetSet('root_folder_id', $rootFolderId);
                    
                    // check if it has rememberMe :
                    if ($request->getPost('rememberme') == 1 ) {
                        $this->getSessionStorage()
                             ->setRememberMe(1);
                        // set storage again
                        $this->getAuthService()->setStorage($this->getSessionStorage());
                    }
                    
                    $this
                        ->getAuthService()
                        ->getStorage()
                        ->write(
                                $request->getPost('username')
                    );
                    
                    $referrer = $request->getPost('referrer');
                    
                    $convert = $this->getServiceLocator()->get('ConvertParams');
                    
                    if (($referrer) && ! empty($referrer)) {
                        
                        $urlArray = explode('/', $referrer);
                            
                        if ($urlArray[3] == 'secure') {
                            
                            if ($urlArray[5]) {
                                
                                $decodedPersonId = $convert->paramDecrypt($urlArray[5]);
                                $resourceStringArray = $convert
                                                            ->stringToArray(
                                                                    $decodedPersonId
                                );
                                
                                if ($resourceStringArray['tab_type'] 
                                                    === $constantArray['TASK_RESOURCE']) {      

                                    if ($resourceStringArray['action']
                                        === $constantArray['resourceAction']['UPDATE_ACTION']) {

                                        if (isset($resourceStringArray['child_resourceid'])) {
                                            
                                            // check if there exist permission for the 
                                            // given user id and the provided resource id.
                                            $isSharedStatus = $this
                                                                ->getUserResourceTable()
                                                                ->isResourceShared(
                                                        $userId, 
                                                        $resourceStringArray['resourceid']
                                            );
                                            
                                            $isOwnerStatus = $this
                                                                ->getUserResourceTable()
                                                                ->isOwner(
                                                        $userId, 
                                                        $resourceStringArray['resourceid']
                                            );
                                                        
                                            if ($isSharedStatus || $isOwnerStatus) {
                                                
                                                return $this
                                                            ->redirect()
                                                            ->toUrl(
                                                                    $request
                                                                        ->getPost(
                                                                                'referrer'
                                                                        )
                                                );
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                else {
                    
                    $userRecord = $this
                                    ->getUsersTable()
                                    ->checkIfRecordInUnactiveState(
                                            $request->getPost('username'), 
                                            $request->getPost('password')
                    );
                    
                    if ($userRecord) {
                        
                        $userIdArray = array(
                                                'userid' => $userRecord['userid']
                        );
                    
                        $encryptedUserId = $this->createEncodedString($userIdArray);
                        
                        $this->flashmessenger()->addMessage(
                                array(
                                    'isRecordInUnactiveState'   => TRUE,
                                    'encryptedUserId'           => $encryptedUserId,
                                    'base_url'                  => $constantArray['base_url']
                                )
                        );
                    }
                }
               
            }
        }
        
        return $this->redirect()->toRoute($redirect);
    }
    
	/**
     * Clears all flash messages, session data and identity from auth service
     *
     * @return object(Zend\Http\PhpEnvironment\Response)
     */
    public function logoutAction()
    {
        // Assign all the group of constants to constArray variable.
        $constantArray = $this->config;

		$isLoggedIn = $constantArray['IS_LOGGED_IN_NO'];
		
		// check if the user is logged in and has its identity in session
        if ($this->getAuthService()->hasIdentity()) {
		
			// removes/unsets data from session using session service manager
            $this->getSessionStorage()->forgetMe();
            $this->getAuthService()->clearIdentity();
            $this->flashmessenger()->addMessage("You've been logged out.");
			$isLoggedIn = $constantArray['IS_LOGGED_IN_YES'];
			$session = new Container($constantArray['SESSION_CONTAINER_NAME']);
			$session->offsetSet('is_logged_in', $constantArray['IS_LOGGED_IN_NO']);
			$session->offsetSet('user_id', '');
            $session->offsetSet('email_login', '');
        }
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
        
		// redirect user to the login page.
        return $this->redirect()->toRoute('login');
    }
    
    /**
     * Function to create an encoded string from the array provided as input.
     * 
     * @param Array $inputArray
     * @return String encoded string on success and false on failure.
     */
	public function createEncodedString($inputArray)
    {
        if (is_array($inputArray)) {
            $convert = $this->getServiceLocator()->get('ConvertParams');
            $string = $convert->arrayToString($inputArray);
            return $convert->paramEncrypt($string);
        }
        return false;
    }
    
    /**
     * Function to decode a string and convert it into an array.
     * 
     * @param String $encryptedString
     * @return Array with required pair of key and value.
     */
	public function createDecodedArray($encryptedString)
    {
        if ($encryptedString) {
            $convert = $this->getServiceLocator()->get('ConvertParams');
            $decodedPersonId = $convert->paramDecrypt($encryptedString);
            return $convert->stringToArray($decodedPersonId);
        }
        return false;
    }
    
    /**
     * Get service locater instance of getUserResourceTable
     * 
     * @return service manager instance for getUserResourceTable
     */
	public function getUserResourceTable()
	{
		// check if usersTable previously defined.
		if (! $this->userResourceTable) {
			$sm = $this->getServiceLocator();
			$this->userResourceTable = $sm->get('Secure\Model\UserResourceTable');
		}
		return $this->userResourceTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     * 
     * @return instance instance of UsersTable model.
     * 
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (! $this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
    
    /**
     * Apply strip tags to the provided input
     *
     * @return filtered value else return false.
     */
	public function filterFormElements($valueToFilter = '')
	{
		// instantiate the strip tag filter
		$stripTagFilter   = new StripTags();
		$StringTrimFilter = new StringTrim();
		if ($valueToFilter) {
			return $stripTagFilter->filter($StringTrimFilter->filter($valueToFilter));
		}
		return false;
	}
    
    /**
     * Get an instance of requestTrackerTable by using service manager.
     *
     * @return service manager instance for request_tracker table.
     */
	public function getRequestTrackerTable()
	{
		// check if usersTable previously defined.
		if (! $this->requestTrackerTable) {
			$sm = $this->getServiceLocator();
			$this->requestTrackerTable = $sm->get('Secure\Model\RequestTrackerTable');
		}
		return $this->requestTrackerTable;
	}
}