<?php

//module/Secure/src/Secure/Model/ContactsTable.php

/**************
	Purpose: Model functions for accessing the contacts table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Adapter\Adapter;

class ContactsTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
    
	/**
     * @var dbSql
     */
    protected $dbSql;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
        $this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Function to get the calendar name from the given calendar id.
     * 
     * @param int $calendar
     * @return boolean record(object) on success and false on failure.
     * 
     */
	public function getContactsNameFromId($contactid)
    {
		$rowset = $this->tableGateway->select(array('contact_groupid' => $contactid));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Update a contact record by contact id.
     * 
     * @param int $contactid
     * @param int $task_name
     * @return boolean true on success and false on failure.
     * 
     */
	public function updateContactsRecordById($contactid, $task_name)
    {
        $updataUsers = array(
            'contact_group_name' =>  $task_name
        );
        
         $updateResource = array(
			'description'   => $task_name,
             'updated'      =>  date('Y-m-d H:i:s')
		);
        
        try {
            $this->tableGateway->update(
                $updataUsers, 
                array('contact_groupid' => $contactid)
            );
            
             // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $contactid,
                            'type'          => 'contact_group'
                        )
            );
            return true;
        } catch (Exception $exc) {
            return false;
        }
        // system has undergone an unknown exception.
    }
}