<?php

//module/Secure/src/Secure/Model/TaskTable.php

/**************
	Purpose: Model functions for accessing the Task table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Adapter\Adapter;

class TaskTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	private $sessionInstance;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
	
	/**
     * @var form
     */
	protected $eventsTable;
	protected $dbSql;
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Model function to all tasks for the given task group.
     * @param  int $resourceid
     */
    public function getAllTasks($resourceid)
    {
        if ($resourceid) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT *
                    FROM resource AS t1
                    LEFT JOIN task AS t2
                    ON ( t1.fk_type_recid = t2.taskid)
                    WHERE t1.fk_resourceid_parent = $resourceid  
                    AND t1.type='task' AND t1.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
    /**
     * Model function to insert user register information into the database.
     * @param  Users $user
     */
    public function saveTask($insertMedicationData, $userId)
    {
        $medicationData = array(
            'task_description'      => $insertMedicationData['task_description'],
            'due_date'              => $insertMedicationData['due_date'],
            'follow_up_date'        => $insertMedicationData['follow_up_date'],
            'priority'              => $insertMedicationData['priority'],
            'person_responsibile'   => $insertMedicationData['person_responsibile'],
            'task_notes'            => $insertMedicationData['task_notes'],
            'is_completed'          => $insertMedicationData['is_completed'],
            'created'               => date('Y-m-d H:i:s'),
            'userid_created'        => $userId
        );
		try {
            
			$this->tableGateway->insert($medicationData);
        
            $lastInsertIdPerson = $this->tableGateway->lastInsertValue;
            $resourceData = array(
                'type'					=>	'task',
                'description'           =>  $insertMedicationData['task_description'],
                'fk_type_recid'			=> 	$lastInsertIdPerson,
                'fk_userid_owner'  		=> 	$userId,
                'fk_resourceid_parent' 	=>  $insertMedicationData['parent_resource_id'],
                'created'  				=>  date('Y-m-d H:i:s'),
                'userid_created' 		=>  $userId,
                'updated'  				=>  date('Y-m-d H:i:s')
            );

            $resourceInstance = new TableGateway('resource', $this->dbSql);
            $resourceInstance->insert($resourceData);
            
            $lastInsertIdTask = $resourceInstance->lastInsertValue;
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * Model function to update a task record in database.
     * @param  Users $user
     */
    public function updateTask($insertTaskData, $user_id)
    {
        $taskData = array(
            'task_description'      => $insertTaskData['task_description'],
            'due_date'              => $insertTaskData['due_date'],
            'follow_up_date'        => $insertTaskData['follow_up_date'],
            'priority'              => $insertTaskData['priority'],
            'person_responsibile'   => $insertTaskData['person_responsibile'],
            'task_notes'            => $insertTaskData['task_notes'],
            'task_assigned_to'      => $insertTaskData['assign_task_to'],
            'is_completed'          => $insertTaskData['is_completed'],
            'created'               => date('Y-m-d H:i:s'),
            'userid_created'        => $user_id
        );
        
        $updateResource = array(
			'description'   => $insertTaskData['task_description'],
            'updated'  		=>  date('Y-m-d H:i:s')
		);
        
		try {
			$this->tableGateway->update(
                    $taskData, 
                    array('taskid' => $insertTaskData['taskid'])
            );
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $insertTaskData['taskid'],
                            'type'          => 'task'
                        )
            );
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * Model function to get the task record by ID.
     * @param  int $taskId
     */
    function getTaskRecordById($taskId) {
        
        $rowset = $this->tableGateway->select(array('taskid' => $taskId));
        $row = $rowset->current();
        if ($row) {
           return $row;
        }
        return false;
    }
    
    /**
     * Delete user permission.
     * 
     * @param type $userResourceId
     * @return boolean tru on success and false on failure
     */
    public function deleteTask($taskId) {
        
        if ($taskId) {
             if ($this->tableGateway->delete(array('taskid' => $taskId))) {
                 $resourceInstance = new TableGateway('resource', $this->dbSql);
                 if ($resourceInstance->delete(
                         array('fk_type_recid' => $taskId, 'type'=>'task'))) {
                     return true;
                 }
             }
        }
        return false;
    }
}