<?php

//module/Secure/src/Secure/Model/EventsTable.php

/**************
	Purpose: Model functions for accessing the events table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Adapter\Adapter;

class TasksTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
	/**
     * @var dbSql
     */
    protected $dbSql;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
        $this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Function to get the calendar name from the given calendar id.
     * 
     * @param int $calendar
     * @return boolean
     * 
     */
	public function getTasksNameFromId($taskid)
    {
		$rowset = $this->tableGateway->select(array('task_groupid' => $taskid));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Function to update the calendar name using the given calendar id.
     * 
     * @param int $calendarid, string $calendar_name
     * @return boolean true on success and false on failure.
     * 
     */
	public function updateTasksRecordById($taskid, $task_name)
    {
        $updataUsers = array(
            'task_group_name' =>  $task_name
        );
        
        $updateResource = array(
			'description'   =>  $task_name,
            'updated'  		=>  date('Y-m-d H:i:s')
		);
        
        try {
            $this->tableGateway->update($updataUsers, array('task_groupid' => $taskid));
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $taskid,
                            'type'          => 'task_group'
                        )
            );
            return true;
        } catch (Exception $exc) {
            return false;
        }
        // system has undergone an unknown exception.
    }
}