<?php
//module/Secure/src/Secure/Model/UsersTable.php

/**************
	Purpose: Model functions for accessing the users table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where;
use Zend\Db\Sql\Select;
use Zend\Db\ResultSet\ResultSet;

class UsersTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	/**
     * @var sessionInstance
     */
	private $sessionInstance;
	
	/**
     * @var eventsTable
     */
	protected $eventsTable;
	
	/**
     * @var dbSql
     */
	protected $dbSql;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;

    /*
     * Defined constants for the resource types
     */
    const RESOURCE_TABLE            = 'resource';
    const RESOURCE_TYPE_TABLE       = 'resource_type';
    const FOLDER_TABLE              = 'folders';
    
    const FOLDER_RESOURCE           = 'folder';
    const FOLDER_DESC               = 'Profiles';
    
    const PERSON_RESOURCE           = 'person';
    
    const HEALTH_AND_MEDICAL_RESOURCE   = 'health&medical';
    const HEALTH_AND_MEDICAL_DESC       = 'Health and Medical';
    
    const FINANCIAL_RESOURCE            = 'financial';
    const FINANCIAL_DESC                = 'Financial';
    
    const LEGAL_RESOURCE                = 'legal';
    const LEGAL_DESC                    = 'Legal';
    
    const POST_RESOURCE                 = 'post';
    const EVENT_RESOURCE                = 'event';
    const TASK_RESOURCE                 = 'task';
    const DOCUMENT_RESOURCE             = 'document';
    const CONTACT_RESOURCE              = 'contact';
    const MEDICATION_RESOURCE           = 'medication';

    /**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }

	/**
     * Model function to insert user register information into the database.
     * @param  Users $user
     */
    public function saveUser($user, $isManual = 'no')
    {
		$usersData = array(
			'password'  			=> md5($user['password']),
			'email_login'  			=> $user['loginEmail'],
			'terms_conditions_agree'=> ($user['termsAgree'] !== '') ? 1 : -1,
            'activation_code'       => $user['activationCode'],
			'created'  				=> date('Y-m-d H:i:s'),
            'rec_status'            => -1
		);
        
        if ($isManual == 'yes') {
            $usersData['rec_status'] = 1;
        }
        
        $folderInstance = new TableGateway(self :: FOLDER_TABLE, $this->dbSql);
        $resourceInstance = new TableGateway(self :: RESOURCE_TABLE, $this->dbSql);
        $resourceTypeInstance = new TableGateway(self :: RESOURCE_TYPE_TABLE, $this->dbSql);
        $personInstance = new TableGateway(self :: PERSON_RESOURCE, $this->dbSql);
        
        $connection = $this->dbSql->getDriver()->getConnection();
		
		try {
            $connection->beginTransaction();
			$this->tableGateway->insert($usersData);
			$lastInsertIdUser = $this->tableGateway->lastInsertValue;
            
            $folderData = array(
                'folder_name'       =>  self :: FOLDER_DESC,
                'created'  			=>  date('Y-m-d H:i:s'),
				'userid_created' 	=>  $lastInsertIdUser
            );
            
            $folderInstance->insert($folderData);
            $lastInsertIdFolder = $folderInstance->lastInsertValue;
            
            $resourceDataFolder = array(
				'type'				  =>	self :: FOLDER_RESOURCE,
				'fk_type_recid'		  => 	$lastInsertIdFolder,
                'description'         =>    self :: FOLDER_DESC,
				'fk_resourceid_parent'=> 	0,
				'fk_userid_owner'  	  => 	$lastInsertIdUser,
				'created'  			  => 	date('Y-m-d H:i:s'),
				'userid_created' 	  => 	$lastInsertIdUser,
                'updated'  			  => 	date('Y-m-d H:i:s')
			);
			$resourceInstance->insert($resourceDataFolder);
            
            $lastInsertIdFolderResource = $resourceInstance->lastInsertValue;
            
            $personData = array(
				'fk_userid'			=>	$lastInsertIdUser,
				'first_name'		=> 	$user['firstName'],
				'last_name'  		=> 	$user['lastName'],
				'created'  			=>  date('Y-m-d H:i:s'),
				'userid_created' 	=>  $lastInsertIdUser
			);
			
			$personInstance->insert($personData);
			$lastInsertIdPerson = $personInstance->lastInsertValue;
            
			$resourceDataPerson = array(
				'type'				  =>	self :: PERSON_RESOURCE,
				'fk_type_recid'		  => 	$lastInsertIdPerson,
                'description'         =>    $user['firstName'] . ' ' . $user['lastName'],
				'fk_resourceid_parent'=> 	$lastInsertIdFolderResource,
				'fk_userid_owner'  	  => 	$lastInsertIdUser,
				'created'  			  => 	date('Y-m-d H:i:s'),
				'userid_created' 	  => 	$lastInsertIdUser,
                'updated'  			  => 	date('Y-m-d H:i:s')
			);
            
			$resourceInstance->insert($resourceDataPerson);
            $lastInsertIdPersonResource = $resourceInstance->lastInsertValue;
            
            $resourceTypeData = array(
                'resourceid'        =>  $lastInsertIdPersonResource,
                'type'              =>  self :: POST_RESOURCE,
                'sort_order'        =>  '1',
                'created'           =>  date('Y-m-d H:i:s'),
				'userid_created'    =>  $lastInsertIdUser,
                'updated'  			=>  date('Y-m-d H:i:s')
			);
            
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type']       = self :: EVENT_RESOURCE;
            $resourceTypeData['sort_order'] = '2';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: TASK_RESOURCE;
            $resourceTypeData['sort_order'] = '3';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '4';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: CONTACT_RESOURCE;
            $resourceTypeData['sort_order'] = '5';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceDataHealth = array(
				'type'					=>	self :: FOLDER_RESOURCE,
                'description'           =>  self :: HEALTH_AND_MEDICAL_DESC,
				'fk_userid_owner'  		=> 	$lastInsertIdUser,
				'fk_resourceid_parent' 	=>  $lastInsertIdPersonResource,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $lastInsertIdUser,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance->insert($resourceDataHealth);
            
            $resourceTypeData['resourceid'] = $resourceInstance->lastInsertValue;
            $resourceTypeData['type'] = self :: MEDICATION_RESOURCE;    
            $resourceTypeData['sort_order'] = '1';
            $resourceTypeInstance->insert($resourceTypeData);
            
            
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '2';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceDataFinancial = array(
				'type'					=>	self :: FOLDER_RESOURCE,
                'description'           =>  self :: FINANCIAL_DESC,
				'fk_userid_owner'  		=> 	$lastInsertIdUser,
				'fk_resourceid_parent' 	=>  $lastInsertIdPersonResource,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $lastInsertIdUser,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance->insert($resourceDataFinancial);
            
            $resourceTypeData['resourceid'] = $resourceInstance->lastInsertValue;
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '1';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceDataLegal = array(
				'type'					=>	self :: FOLDER_RESOURCE,
                'description'           =>  self :: LEGAL_DESC,
				'fk_userid_owner'  		=> 	$lastInsertIdUser,
				'fk_resourceid_parent' 	=>  $lastInsertIdPersonResource,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $lastInsertIdUser,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
            
			$resourceInstance->insert($resourceDataLegal);
            
            $resourceTypeData['resourceid'] = $resourceInstance->lastInsertValue;
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '1';
            $resourceTypeInstance->insert($resourceTypeData);
            
            // return the last user id inserted
            $connection->commit();
			return $lastInsertIdUser;
		}
		catch (\Exception $e) {
            $connection->rollback();
            // Log errors as needed.
			return false;
		}
		// system has undergone an unknown exception.
    }
	
	/**
     * Model function to insert user register information into the database.
     * @param  Users $user
     */
    public function updateUser($user, $id=0)
    {	
		$updataUsers = array();
		if ($user['password'] == '') {
			$updataUsers = array(
				'email_login'  		=> $user['userEmail'],
				'userid_updated' 	=> $id
			);
		} else {
			$updataUsers = array(
				'password'  		=> md5($user['password']),
				'email_login'  		=> $user['userEmail'],
				'userid_updated' 	=> $id
			);
		}
		
		$updatePerson = array(
			'first_name' => $user['userFirstName'],
			'last_name' => $user['userLastName']
		);
        
        $updateResource = array(
			'description'   => $user['userFirstName'] . ' ' . $user['userLastName'],
            'updated'       => date('Y-m-d H:i:s')
		);
		
		try {
			$this->tableGateway->update($updataUsers, array('userid' => $id));
			$personInstance = new TableGateway('person', $this->dbSql);
			$personInstance->update($updatePerson, array('fk_userid' => $id));
            
            $personRecord = $this->getPersonIdUsingUserId($id);
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                            $updateResource, 
                            array(
                                'fk_userid_owner'   => $id,
                                'type'              => 'person',
                                'fk_type_recid'     => $personRecord->personid
                            )
            );
			return true;
		}
		catch (\Exception $e) {
            return false;
		}
		// system has undergone an unknown exception
	}
    
    /**
     * Model function to activate an user using the sent secrect code.
     * 
     * @param string $secretCode
     * @return boolean true if success and false if failure.
     */
    public function activateUser($secretCode)
    {
       $status = $this->isEmailAddressActive($secretCode);
       
       if (!$status) {
            $set = array(
                'rec_status'    => 1
            );

            $where = array(
                'activation_code'   =>  $secretCode
            );

            try {
                $this->tableGateway->update($set, $where);
                return true;
            }
            catch (Exception $e) {
                return false;
            }
       }
       else {
           return false;
       }
    }
    
    /**
     * Model function to check if the email address is active.
     * 
     * @param string $secretCode
     * @return boolean true if success and false if failure.
     */
    public function isEmailAddressActive($secretCode) {
        // get email address for the secret code
        $rowset = $this->tableGateway->select(array('activation_code' => $secretCode));
        $row = $rowset->current();
        if ($row) {
            // check if the email address is already active
            $rowset = $this
                        ->tableGateway
                        ->select(
                                array(
                                    'email_login' => $row['email_login'], 
                                    'rec_status' => 1
                                )
            );
            
            $row = $rowset->current();
            if ($row) {
                return true;
            }
            else {
                return false;
            }
        }
        return true;
    }
	
    /**
     * Model function to get all user details with the help of userid.
     * 
     * @param int $userId
     * @return boolean
     */
	public function getUser($userId)
    {
		try {
			$sql = new Sql($this->tableGateway->getAdapter());
			$select = $sql->select();
			$where = new Where();
			//$where->equalTo('users.rec_status', '1');
			$where->equalTo('userid', $userId);

			$select
				->from('users')
				->join(
					array('C' => 'person'),
					'userid = C.fk_userid'
			);
			$select->where($where);
			$rowsetPerson = $this->tableGateway->selectWith($select);
			return $rowsetPerson->current();
		}
		catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * 
     * @param type $userid
     * @return boolean
     * 
     */
    function getRootFolderIdByUserid($userid = '')
    {
        if ($userid) {
            
            $personInstance = new TableGateway('person', $this->dbSql);
            $rowset = $personInstance->select(array('fk_userid' => $userid));
            
            $row = $rowset->current();
            
            $personInstance = new TableGateway('resource', $this->dbSql);
            $rowset = $personInstance->select(
                                    array(
                                        'fk_type_recid' => $row['personid'],
                                        'type'          => 'person'
                                    )
            );
            
            $row = $rowset->current();
            if ($row['fk_resourceid_parent']) {
                return $row['fk_resourceid_parent'];
            }
        }
        return false;
    }
	
    /**
     * Function to ger user record on the basis of the email id.
     * 
     * @param string $emailId
     * @return boolean false on failure and user record on success.
     */
	public function getUserbyEmail($emailId)
    {
        $rowset = $this->tableGateway->select(array('email_login' => $emailId));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Function to ger user record on the basis of the email id.
     * 
     * @param string $emailId
     * @return boolean false on failure and user record on success.
     */
	public function getUserIdbyEmail($emailId)
    {
        $rowset = $this->tableGateway->select(
            array(
                'email_login' => $emailId, 
                'rec_status' => 1
            )
        );
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
	
    /**
     * Function for resetting the password for an registered and activated user.
     * 
     * @param int $userId
     * @param string $password
     * @return boolean
     */
	public function resetPassword($userId = '', $password = '')
    {
		if ($userId && $password) {
			$updatePassword = $this
								->tableGateway
								->update(
									array('password' => $password), 
									array('userid' => $userId, 'rec_status' => 1)
			);
            
			if ($updatePassword) {
				// 'email_address' => $emailAddress
                return $this->getUser($userId);
			}
		}
		return false;
    }
    
    /**
     * Function to get the person id with the help of the userid.
     * 
     * @param int $userId
     * @return boolean
     */
    public function getPersonIdUsingUserId($userId)
    {
        $resourceInstance = new TableGateway('person', $this->dbSql);
        $rowset = $resourceInstance->select(array('fk_userid' => $userId));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Check if the given email id exists in the system.
     * 
     * @param string $emailId
     * @return boolean true id exists false if it does not.
     * 
     */
    public function ifEmailIdExists($emailId)
    {
        $rowset = $this->tableGateway->select(
                                        array(
                                            'email_login' => $emailId,
                                            'rec_status'  => 1
                                        )
        );
        
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Check if email id exists in an inactive state.
     * 
     * @param string $emailId
     * @return boolean
     */
    public function ifEmailIdExistsInactive($emailId)
    {
        $rowset = $this->tableGateway->select(
                                        array(
                                            'email_login' => $emailId,
                                            'rec_status'  => -1
                                        )
        );
        
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Get the details of user by activation code.
     * 
     * @param type $activationCode
     */
    public function getUserDetailByActivationCode($activationCode)
    {
        $rowset = $this->tableGateway->select(
                                        array(
                                            'activation_code' => $activationCode
                                        )
        );
        
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
    * Function to check if the user record is present in unactivated state.
    * 
    * @param type $activationCode
    */
    public function checkIfRecordInUnactiveState($userEmail, $password)
    {
        $rowset = $this->tableGateway->select(
                                                array(
                                                    'email_login' => $userEmail,
                                                    'password'    => md5($password),
                                                    'rec_status'  => -1
                                                )
        );
        
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Function to get all inactive records from the users table
     * 
     * $whereConditionArray
     */
    public function getAllInactiveRecords($whereConditionArray)
    {
        if ($whereConditionArray) {
            
            $rowset = $this->tableGateway->select($whereConditionArray);
            
            if ($rowset) {
                
                return $rowset;
            }
        }
        
        return false;
    }
}